@@ -4,12 +4,13 @@ import Image
 import gtk.glade, gtk, gobject, gtk.gdk, traceback
 import batchEditor
 import recipeManager
-import exporters.printer as printer
+from exporters.printer import get_print_manager
 import prefs, prefsGui, shopgui, reccard, fnmatch, tempfile
 import exporters, importers
 from exporters.exportManager import get_export_manager
 from importers.importManager import get_import_manager
 import convert, version
+from gtk_extras import fix_action_group_importance
 from gtk_extras import ratingWidget, WidgetSaver, mnemonic_manager
 from gtk_extras import dialog_extras as de
 from gtk_extras import treeview_extras as te
@@ -22,6 +23,7 @@ import locale, gettext
 from timer import show_timer
 _ = gettext.gettext
 from defaults.defaults import lang as defaults
+from defaults.defaults import get_pluralized_form
 import plugin_loader, plugin, plugin_gui
 from threadManager import get_thread_manager, get_thread_manager_gui, SuspendableThread
 from zipfile import BadZipfile
@@ -126,7 +128,7 @@ class GourmetApplication:
     # Setup preferences system
     def setup_prefs (self):
         self.conf = []        
-        self.prefs = prefs.Prefs()
+        self.prefs = prefs.get_prefs()
         self.prefsGui = prefsGui.PreferencesGui(
             self.prefs,
             buttons={'clear_remembered_optional_button':lambda *args: self.forget_remembered_optional_ingredients()}
@@ -204,7 +206,11 @@ class GourmetApplication:
         # initiate autosave stuff autosave every 3 minutes
         # (milliseconds * 1000 milliseconds/second * 60
         # seconds/minute)
-        gobject.timeout_add(1000*60*3,lambda *args: self.rd.save() or True)
+        def autosave ():
+            self.rd.save()
+            return True
+        AUTOSAVE_EACH_N_MINUTES = 2
+        gobject.timeout_add(1000*60*AUTOSAVE_EACH_N_MINUTES,autosave)
         # connect hooks to modify our view whenever and
         # whenceever our recipes are updated...
         self.rd.modify_hooks.append(self.update_attribute_models)
@@ -219,7 +225,7 @@ class GourmetApplication:
     def setup_shopping (self):
         """Setup shopping related stuff"""
         #self.create_rmodel(self.rd.recipe_table)
-        self.sl = shopgui.ShopGui(self, conv=self.conv)
+        self.sl = shopgui.ShopGui()
         self.sl.hide()
 
     # Methods for keeping track of open recipe cards...
@@ -264,6 +270,7 @@ class GourmetApplication:
         self.merged_go_menus = {}
 
     def add_uimanager_to_manage (self, id, uimanager, menu_root_name):
+        fix_action_group_importance(self.goActionGroup)
         uimanager.insert_action_group(self.goActionGroup,0)
         uimanager.add_ui_from_string(self.shared_go_menu%{'name':menu_root_name})
         self.uimanagers[id] = [uimanager,menu_root_name,
@@ -429,9 +436,9 @@ class GourmetApplication:
             for a in authors:
                 sublabel += '\n%s'%a
             if translator:
-                sublabel += _('\nTranslated by: %s')%translator
+                sublabel += '\n' + _('Translated by')+': %s'%translator
             if website:
-                sublabel += _('\nWebsite: %s')%website
+                sublabel += '\n'+_('Website')+': %s'%website
             import xml.sax.saxutils
             de.show_message(label=xml.sax.saxutils.escape('%s %s'%(appname,myversion)),
                             sublabel=xml.sax.saxutils.escape(sublabel)) #new line that leaves strings as they are.
@@ -732,18 +739,13 @@ class ImporterExporter:
     def print_recs (self, *args):
         debug('printing recipes',3)
         recs = self.get_selected_recs_from_rec_tree()
-        renderer = printer.RecRenderer(self.rd, recs,
-                                       dialog_title=gettext.ngettext('Print %s recipe',
-                                                                     'Print %s recipes',
-                                                                     len(recs))%len(recs),
-                                       dialog_parent = self.app,
-                                       change_units = self.prefs.get('readableUnits',True)
-                                       )
-        #tm = get_thread_manager()
-        #tmg = get_thread_manager_gui()
-        #tm.add_thread(renderer)
-        #tmg.register_thread_with_dialog(_('Print recipes'),renderer)
-        #tmg.show()
+        printManager = get_print_manager()
+        printManager.print_recipes(
+            self.rd,
+            recs,
+            parent=self.app,
+            change_units=self.prefs.get('readableUnits',True)
+            )
 
     def import_webpageg (self, *args):
         self.importManager.offer_web_import(parent=self.app.get_toplevel())
@@ -775,12 +777,16 @@ class StuffThatShouldBePlugins:
         rr=self.get_selected_recs_from_rec_tree()
         #r = self.recTreeSelectedRec()
         for r in rr:
-            if r.servings and r.servings != "None":
-                debug("servings=%s"%r.servings,5)
-                serv = de.getNumber(default=float(r.servings),
-                                    label=_("Number of servings of %s to shop for")%r.title,
-                                    parent=self.app.get_toplevel())
-                if serv: mult = float(serv)/float(r.servings)
+            if r.yields and r.yields != "None":
+                debug("yields=%s"%r.yields,5)
+                serv = de.getNumber(default=float(r.yields),
+                                    label=_("Number of %(unit)s of %(title)s to shop for")%{
+                        'title':r.title,
+                        'unit':get_pluralized_form(r.yield_unit,2),
+                        },
+                                    parent=self.app.get_toplevel()
+                                    )
+                if serv: mult = float(serv)/float(r.yields)
                 else:
                     debug('getNumber cancelled',2)
                     return
@@ -868,7 +874,6 @@ ui = '''<ui>
   <menu name="Tools" action="Tools">
     <placeholder name="StandaloneTool">
     <menuitem action="Timer"/>
-    <menuitem action="FieldEditor"/>
     </placeholder>
     <separator/>
     <placeholder name="DataTool"/>
@@ -920,6 +925,7 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
                           rg=self,
                           editable=False)
         self.setup_database_hooks()        
+        fix_action_group_importance(self.search_actions)
         self.ui_manager.insert_action_group(self.search_actions,0)
         self.setup_main_window()
         self.window.add_accel_group(self.ui_manager.get_accel_group())
@@ -1004,9 +1010,23 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
                                        tab_label=gtk.Label(_('Search recipes')))
         self.main.add(self.main_notebook)
         self.recipe_index_interface.show()
+
         self.main_notebook.show(); self.main_notebook.set_show_tabs(False)
+
+        # Set up right-clicking again
+        self.rectree.connect('popup-menu',self.rectree_popup)
+        def popcb (tv, event):
+            if event.button==3:
+                self.rectree_popup(tv,event)
+                return True
+        self.rectree.connect('button-press-event',popcb)
         self.main.show()
 
+    def rectree_popup (self, tv, event, *args):
+        menu = self.ui_manager.get_widget("/RecipeIndexMenuBar/Actions/").get_submenu()
+        menu.popup(None,None,None,event.button,event.time)
+        return True
+
     def setup_actions (self):
         self.ui_manager = gtk.UIManager()
         self.ui_manager.add_ui_from_string(ui)
@@ -1067,12 +1087,6 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             ('Tools',None,_('_Tools')),
             ('Timer',None,_('_Timer'),
              None,_('Show timer'),lambda *args: show_timer()),
-            ('FieldEditor',None,_('_Field Editor'),
-             None,_('Edit values for any field across multiple recipes at a time'),
-             lambda *args: self.showValueEditor()
-             ),
-            #('UnitConverter',None,_('_Unit Converter'),
-            # None,_('Calculate unit conversions'),self.showConverter),
             ])
 
         
@@ -1080,8 +1094,12 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             ('GoRecipeIndex',None,_('Recipe _Index'),
              None,_('Searchable index of recipes in the database.'),self.present)]
                                        )
+        fix_action_group_importance(self.onSelectedActionGroup)
         self.ui_manager.insert_action_group(self.onSelectedActionGroup,0)
+        fix_action_group_importance(self.mainActionGroup)
+        fix_action_group_importance(self.mainActionGroup)
         self.ui_manager.insert_action_group(self.mainActionGroup,0)
+        fix_action_group_importance(self.toolActionGroup)
         self.ui_manager.insert_action_group(self.toolActionGroup,0)
         self.add_uimanager_to_manage(-1,self.ui_manager,'RecipeIndexMenuBar')
 
@@ -1309,7 +1327,8 @@ class RecGui (RecIndex, GourmetApplication, ImporterExporter, StuffThatShouldBeP
             self.hide_progress_dialog()            
         d=de.MessageDialog(label=label,
                            sublabel=sublabel,
-                           cancel=False
+                           cancel=False,
+                           modal=False
                            )
         b = gtk.Button(stock=gtk.STOCK_JUMP_TO)
         b.connect('clicked',lambda *args: launch_url(url))
