@@ -3,8 +3,9 @@
 #sys.path.append('/usr/share/')
 ## End for testing - DELETE ME
 import shutil
+import types
 import os.path
-from gourmet.gdebug import debug, TimeAction
+from gourmet.gdebug import debug, TimeAction, debug_decorator
 import re, pickle, string, os.path, string, time
 from gettext import gettext as _
 import gourmet.gglobals as gglobals
@@ -58,10 +59,12 @@ def fix_colnames (dict, *tables):
 def make_simple_select_arg (criteria,*tables):
     args = []
     for k,v in fix_colnames(criteria,*tables).items():
+        if type(v)==str:
+            v = unicode(v)
         if type(v)==tuple:
             operator,value = v
-            if type(v)==str:
-                v = unicode(v)
+            if type(value)==str:
+                value = unicode(value)
             if operator=='in':
                 args.append(k.in_(value))
             elif hasattr(k,operator):
@@ -98,8 +101,8 @@ def make_order_by (sort_by, table, count_by=None, join_tables=[]):
                     raise ValueError("No such column for tables %s %s: %s"%(table, join_tables, col))
         if isinstance(col.type, Text):
             # Sort nulls last rather than first using case statement...
-            col = case([(col == None, 'z'*20),
-                        (col == '', 'z'*20),
+            col = case([(col == None, '"%s"'%'z'*20),
+                        (col == '', '"%s"'%'z'*20),
                         ],else_=func.lower(col))
         if direction==1: # Ascending
             ret.append(sqlalchemy.asc(col))
@@ -167,7 +170,7 @@ class RecData (Pluggable):
         debug('Initializing DB connection',1)
         if self.filename:
             self.new_db = not os.path.exists(self.filename)
-            print 'Connecting to file ',self.filename
+            #print 'Connecting to file ',self.filename,'new=',self.new_db
         else:
             self.new_db = True # ??? How will we do this now?
         self.db = sqlalchemy.create_engine(self.url,strategy='threadlocal') 
@@ -279,7 +282,15 @@ class RecData (Pluggable):
                                   Column('source',Text(),**{}),
                                   Column('preptime',Integer(),**{}),
                                   Column('cooktime',Integer(),**{}),
-                                  Column('servings',Float(),**{}),
+                                  # Note: we're leaving servings
+                                  # around as a legacy column... it is
+                                  # replaced by yields/yield_unit, but
+                                  # update is much easier if it's
+                                  # here, and it doesn't do much harm
+                                  # to have it around.
+                                  Column('servings',Float(),**{}), 
+                                  Column('yields',Float(),**{}),                                  
+                                  Column('yield_unit',String(length=32),**{}),
                                   Column('image',Binary(),**{}),
                                   Column('thumb',Binary(),**{}),
                                   Column('deleted',Boolean(),**{}),
@@ -290,6 +301,7 @@ class RecData (Pluggable):
                                   Column('link',Text(),**{}), # A field for a URL -- we ought to know about URLs
                                   Column('last_modified',Integer(),**{}),
                                   ) # RECIPE_TABLE_DESC
+
         class Recipe (object): pass
         self._setup_object_for_table(self.recipe_table,Recipe)
 
@@ -446,30 +458,37 @@ class RecData (Pluggable):
                     stored_info,
                     default_info)
             stored_info = self.fetch_one(self.info_table)            
-        
+    
         ### Code for updates between versions...
-        if not self.new_db:  
-            # Version < 0.11.4 -> version >= 0.11.4... fix up screwed up keylookup_table tables...
-            # We don't actually do this yet... (FIXME)
+        if not self.new_db:
+            sv_text = "%s.%s.%s"%(stored_info.version_super,stored_info.version_major,stored_info.version_minor)
             #print 'STORED_INFO:',stored_info.version_super,stored_info.version_major,stored_info.version_minor
-            if stored_info.version_super == 0 and stored_info.version_major <= 11 and stored_info.version_minor <= 3:
-                self.backup_db()
-                print 'Fixing broken ingredient-key view from earlier versions.'
-                # Drop keylookup_table table, which wasn't being properly kept up
-                # to date...
-                self.delete_by_criteria(self.keylookup_table,{}) 
-                # And update it in accord with current ingredients (less
-                # than an ideal decision, alas)
-                for ingredient in self.fetch_all(self.ingredients_table,deleted=False):
-                    self.add_ing_to_keydic(ingredient.item,ingredient.ingkey)
-
+            # Change from servings to yields! ( we use the plural to avoid a headache with keywords)
+            if (stored_info.version_super == 0 and ((stored_info.version_major <= 14 and stored_info.version_minor <= 7)
+                                                    or
+                                                    (stored_info.version_major < 14)
+                                                    )):
+                print 'Database older than 0.14.7 -- updating',sv_text
+                # Don't change the table defs here without changing them
+                # above as well (for new users) - sorry for the stupid
+                # repetition of code.
+                self.add_column_to_table(self.recipe_table,('yields',Float(),{}))
+                self.add_column_to_table(self.recipe_table,('yield_unit',String(length=32),{}))
+                #self.db.execute('''UPDATE recipes SET yield = servings, yield_unit = "servings" WHERE EXISTS servings''')
+                self.recipe_table.update(whereclause=self.recipe_table.c.servings
+                                       ).values({
+                        self.recipe_table.c.yield_unit:'servings',
+                        self.recipe_table.c.yields:self.recipe_table.c.servings
+                        }
+                                                ).execute()
             if stored_info.version_super == 0 and stored_info.version_major < 14:
+                print 'Database older than 0.14.0 -- updating',sv_text
                 self.backup_db()
                 # Name changes to make working with IDs make more sense
                 # (i.e. the column named 'id' should always be a unique
                 # identifier for a given table -- it should not be used to
                 # refer to the IDs from *other* tables
-                print 'Upgrade from < 0.14'
+                print 'Upgrade from < 0.14',sv_text
                 self.alter_table('categories',self.setup_category_table,
                                  {'id':'recipe_id'},['category'])
                 print 'RECREATE INGREDIENTS TABLE (This could take a while...)'
@@ -485,7 +504,7 @@ class RecData (Pluggable):
             # (These all get added in 0.13.0)
             if stored_info.version_super == 0 and stored_info.version_major <= 12:
                 self.backup_db()                
-                print 'UPDATE FROM < 0.13.0...'
+                print 'UPDATE FROM < 0.13.0...',sv_text
                 # Don't change the table defs here without changing them
                 # above as well (for new users) - sorry for the stupid
                 # repetition of code.
@@ -494,7 +513,7 @@ class RecData (Pluggable):
                 self.add_column_to_table(self.recipe_table,('ingredient_hash',String(length=32),{}))
                 # Add a link field...
                 self.add_column_to_table(self.recipe_table,('link',Text(),{}))
-                print 'Searching for links in old recipe fields...'
+                print 'Searching for links in old recipe fields...',sv_text
                 URL_SOURCES = ['instructions','source','modifications']
                 recs = self.search_recipes(
                     [
@@ -539,6 +558,19 @@ class RecData (Pluggable):
                                 )
                 # Add hash values to identify all recipes...
                 for r in self.fetch_all(self.recipe_table): self.update_hashes(r)
+
+            if stored_info.version_super == 0 and stored_info.version_major <= 11 and stored_info.version_minor <= 3:
+                print 'version older than 0.11.4 -- doing update',sv_text
+                self.backup_db()
+                print 'Fixing broken ingredient-key view from earlier versions.'
+                # Drop keylookup_table table, which wasn't being properly kept up
+                # to date...
+                self.delete_by_criteria(self.keylookup_table,{}) 
+                # And update it in accord with current ingredients (less
+                # than an ideal decision, alas)
+                for ingredient in self.fetch_all(self.ingredients_table,deleted=False):
+                    self.add_ing_to_keydic(ingredient.item,ingredient.ingkey)
+
             for plugin in self.plugins:
                 self.update_plugin_version(plugin,
                                            (current_super,current_major,current_minor)
@@ -839,10 +871,13 @@ class RecData (Pluggable):
         column, following the format for new tables.
         """
         name = table.name; new_col = column_spec[0]; coltyp = column_spec[1]
+        if hasattr(coltyp ,'dialect_impl'):
+            coltyp = coltyp.dialect_impl(self.db.dialect).get_col_spec()
         sql = 'ALTER TABLE %(name)s ADD %(new_col)s %(coltyp)s;'%locals()
         try:
             self.db.execute(sql)
         except:
+            print 'FAILED TO EXECUTE',sql
             print 'Ignoring error in add_column_to_table'
             import traceback; traceback.print_exc()
 
@@ -881,20 +916,18 @@ class RecData (Pluggable):
         del self.metadata.tables[table_name]
         setup_function()
         getattr(self,'%s_table'%table_name).create()
-        for row in self.db.execute('''SELECT %(cols)s FROM %(t)s_temp'''%{
-            't':table_name,
-            'cols':', '.join(cols_to_change.keys()+cols_to_keep)
-            }).fetchall():
-            newdic = {}
-            for k in cols_to_change:
-                newdic[cols_to_change[k]]=getattr(row,k)
-            for c in cols_to_keep:
-                newdic[c] = getattr(row,c)
-            try:
-                self.do_add(getattr(self,'%s_table'%table_name),newdic)
-            except:
-                print 'Trouble inserting newdic,',newdic
-                raise
+        TO_COLS = cols_to_keep[:]
+        FROM_COLS = cols_to_keep[:]
+        for fro,to_ in cols_to_change.items():
+            FROM_COLS.append(fro)
+            TO_COLS.append(to_)
+        stmt = '''INSERT INTO %(t)s (%(to_cols)s)
+        SELECT %(from_cols)s FROM %(t)s_temp
+        '''%{'t':table_name,
+             'from_cols':', '.join(FROM_COLS),
+             'to_cols':', '.join(TO_COLS),
+             }
+        self.db.execute(stmt)        
         self.db.execute('DROP TABLE %s_temp'%table_name)
 
     # Metakit has no AUTOINCREMENT, so it has to do special magic here
@@ -1038,7 +1071,7 @@ class RecData (Pluggable):
                 traceback.print_stack()
         for k,v in recdic.items():
             try:
-                recdic[k]=v.strip()
+                recdic[k]=unicode(v.strip())
             except:
                 pass
 
@@ -1158,9 +1191,18 @@ class RecData (Pluggable):
 
     def add_ings (self, dics):
         """Add multiple ingredient dictionaries at a time."""
-        for d in dics: self.validate_ingdic(d)
+        for d in dics:
+            self.validate_ingdic(d)
+            for k in ['refid','unit','amount','rangeamount','item','ingkey','optional','shopoptional','inggroup','position']:
+                if not k in d:
+                    d[k] = None
         try:
-            self.ingredients_table.insert().execute(*dics)
+            # Warning: this method relies on all the dictionaries
+            # looking identical. validate_ingdic should be taking care
+            # of this for us now, but if parameters change in the
+            # future, this rather subtle bug could well rear its ugly
+            # head again.
+            rp = self.ingredients_table.insert().execute(*dics)
         except ValueError:
             for d in dics: self.coerce_types(self.ingredients_table,d)
             self.ingredients_table.insert().execute(*dics)
@@ -1187,11 +1229,29 @@ class RecData (Pluggable):
                 if dic[k] != v:
                     dic[k] = v
 
+    def commit_fast_adds (self):
+        if hasattr(self,'extra_connection'):
+            self.extra_connection.commit()
+
+    def do_add_fast (self, table, dic):
+        '''Add fast -- return None'''
+        if not hasattr(self,'extra_connection'):
+            self.extra_connection = self.db.connect().connection
+        try:
+            tname = table.name
+            SQL = 'INSERT INTO ' + tname + '('+', '.join(dic.keys()) + ')'
+            SQL += ' VALUES (' +  ", ".join(['?']*len(dic)) + ')'
+            self.extra_connection.execute(SQL,dic.values())
+        except:
+            return self.do_add(table,dic)
+
     def do_add (self, table, dic):
         insert_statement = table.insert()
+        self._force_unicode(dic)
         try:
             result_proxy = insert_statement.execute(**dic)
         except ValueError:
+            print 'Had to coerce types',table,dic
             self.coerce_types(table,dic)
             result_proxy = insert_statement.execute(**dic)
         return result_proxy
@@ -1231,7 +1291,14 @@ class RecData (Pluggable):
     def validate_ingdic (self,dic):
         """Do any necessary validation and modification of ingredient dictionaries."""
         if not dic.has_key('deleted'): dic['deleted']=False
+        self._force_unicode(dic)
 
+    def _force_unicode (self, dic):
+       for k,v in dic.items():
+            if type(v)==str:
+                # force unicode...
+                dic[k]=unicode(v) 
+                
     def do_modify_rec (self, rec, dic):
         """This is what other DBs should subclass."""
         return self.do_modify(self.recipe_table,rec,dic)
@@ -1291,6 +1358,23 @@ class RecData (Pluggable):
             else:
                 print 'Very odd: no match for',ing,'refid:',ing.refid
 
+    def include_linked_recipes (self, recs):
+        '''Handed a list of recipes, append any recipes that are
+        linked as ingredients in those recipes to the list.
+
+        Modifies the list in place.
+        '''
+        import sqlalchemy
+        ids = [r.id for r in recs]
+        extra_ings = self.ingredients_table.select(and_(
+                self.ingredients_table.c.refid,
+                self.ingredients_table.c.recipe_id.in_(ids)
+                )
+                                                  ).execute().fetchall()
+        for i in extra_ings:
+            if i.refid not in ids:
+                recs.append(self.get_referenced_rec(i))
+                
     def get_rec (self, id, recipe_table=None):
         """Handed an ID, return a recipe object."""
         if recipe_table:
@@ -1425,11 +1509,13 @@ class RecData (Pluggable):
         If we are handed a converter interface, we will adjust the
         units to make them readable.
         """
-        amt=self.get_amount(ing,mult)
-        unit=ing.unit
+        amt = self.get_amount(ing,mult)
+        unit = ing.unit
         ramount = None
         if type(amt)==tuple: amt,ramount = amt
         if adjust_units or preferred_unit_groups:
+            if not conv:
+                conv = convert.get_converter()
             amt,unit = conv.adjust_unit(amt,unit,
                                         favor_current_unit=favor_current_unit,
                                         preferred_unit_groups=preferred_unit_groups)
@@ -1497,22 +1583,30 @@ class RecData (Pluggable):
             elif mode==self.AMT_MODE_HIGH: return high # mode==self.AMT_MODE_HIGH
             else:
                 raise ValueError("%s is an invalid value for mode"%mode)
-    
+
+    @pluggable_method
     def add_ing_to_keydic (self, item, key):
         #print 'add ',item,key,'to keydic'
+        # Make sure we have unicode...
+        if type(item)==str: item = unicode(item)
+        if type(key)==str: key = unicode(key)
         if not item or not key: return
+        else:
+            if item: item = unicode(item)
+            if key: key = unicode(key)
         row = self.fetch_one(self.keylookup_table, item=item, ingkey=key)
         if row:
             self.do_modify(self.keylookup_table,row,{'count':row.count+1})
         else:
             self.do_add(self.keylookup_table,{'item':item,'ingkey':key,'count':1})
+        # The below code should move to a plugin for users who care about ingkeys...
         for w in item.split():
             w=str(w.decode('utf8').lower())
-            row = self.fetch_one(self.keylookup_table,word=w,ingkey=key)
+            row = self.fetch_one(self.keylookup_table,word=unicode(w),ingkey=unicode(key))
             if row:
                 self.do_modify(self.keylookup_table,row,{'count':row.count+1})
             else:
-                self.do_add(self.keylookup_table,{'word':w,'ingkey':key,'count':1})
+                self.do_add(self.keylookup_table,{'word':unicode(w),'ingkey':unicode(key),'count':1})
 
     def remove_ing_from_keydic (self, item, key):
         #print 'remove ',item,key,'to keydic'        
@@ -1666,7 +1760,7 @@ class RecipeManager (RecData):
                 return l
         else:
             return None
-            
+
     def ingredient_parser (self, s, conv=None, get_key=True):
         """Handed a string, we hand back a dictionary representing a parsed ingredient (sans recipe ID)"""
         debug('ingredient_parser handed: %s'%s,0)
@@ -1851,6 +1945,25 @@ class dbDic:
         retstr += "}"
         return retstr
 
+    def initialize (self, d):
+        '''Initialize values based on dictionary d
+
+        We assume the DB is known to be empty.
+
+        '''
+        dics = []
+        for k in d:
+            if self.pickle_val:
+                store_v = pickle.dumps(d[k])
+            else:
+                store_v = d[k]
+                if type(store_v) in types.StringTypes:
+                    store_v = unicode(store_v)
+            if type(k) in types.StringTypes:
+                k = unicode(k)
+            dics.append({self.kp:k,self.vp:store_v})
+        self.vw.insert().execute(*dics)
+
     def keys (self):
         ret = []
         for i in self.db.fetch_all(self.vw):
