@@ -77,7 +77,7 @@ class ModalDialog (gtk.Dialog):
         self.sublabel.set_alignment(0,0)
         self.sublabel.set_justify(gtk.JUSTIFY_LEFT)
         self.sublabel.set_use_markup(True)
-        self.sublabel.set_line_wrap(gtk.WRAP_WORD)
+        self.sublabel.set_line_wrap_mode(pango.WRAP_WORD)
         self.sublabel.show()
 
     def setup_buttons (self, cancel, okay):
@@ -114,7 +114,7 @@ class ModalDialog (gtk.Dialog):
         if type(item)==type(""):
             l=gtk.Label(item)
             l.set_selectable(True)
-            l.set_line_wrap(gtk.WRAP_WORD)
+            l.set_line_wrap_mode(pango.WRAP_WORD)
             self.expander_vbox.pack_start(l,
                                           expand=False,
                                           fill=False)
@@ -143,15 +143,17 @@ class MessageDialog (gtk.MessageDialog, ModalDialog):
     """A simple class for displaying messages to our users."""
     
     def __init__ (self, title="", default=None, okay=True, cancel=True, label=False, sublabel=False,
-                  expander=None, message_type=gtk.MESSAGE_INFO, parent=None):
+                  expander=None, message_type=gtk.MESSAGE_INFO, parent=None, modal=True):
         self.message_type=message_type
-        ModalDialog.__init__(self, title=title, default=default, okay=okay, cancel=cancel, label=label, sublabel=sublabel, parent=parent, expander=expander)
+        ModalDialog.__init__(self, title=title, default=default, okay=okay, cancel=cancel, label=label, sublabel=sublabel, parent=parent, expander=expander, modal=modal)
 
     def setup_dialog (self, *args, **kwargs):
         kwargs['type']=self.message_type
         if kwargs.has_key('title'):
+            title = kwargs['title']
             del kwargs['title']
         gtk.MessageDialog.__init__(self, *args, **kwargs)
+        self.set_title(title)
 
     def setup_label (self, label):
         if not is_markup(label):
@@ -191,7 +193,7 @@ class NumberDialog (ModalDialog):
         if label:
             self.label=gtk.Label(label)
             self.label.set_selectable(True)
-            self.label.set_line_wrap(gtk.WRAP_WORD)
+            self.label.set_line_wrap_mode(pango.WRAP_WORD)
             self.label.set_padding(H_PADDING, Y_PADDING)
             self.hbox.add(self.label)
             self.label.show()
@@ -226,7 +228,7 @@ class EntryDialog (ModalDialog):
         self.vbox.add(self.hbox)
         if entryLabel:
             self.elabel=gtk.Label(entryLabel)
-            self.elabel.set_line_wrap(gtk.WRAP_WORD)
+            self.elabel.set_line_wrap_mode(pango.WRAP_WORD)
             self.elabel.set_selectable(True)
             self.elabel.set_alignment(0,0)
             self.hbox.add(self.elabel)
@@ -364,7 +366,7 @@ class ProgressDialog (ModalDialog):
         self.vbox.add(self.detail_label)
         self.detail_label.set_use_markup(True)
         self.detail_label.set_padding(H_PADDING,Y_PADDING)
-        self.detail_label.set_line_wrap(gtk.WRAP_WORD)
+        self.detail_label.set_line_wrap_mode(pango.WRAP_WORD)
         self.vbox.show_all()
         if okay: self.set_response_sensitive(gtk.RESPONSE_OK,False) # we're false by default!
         if not stop: self.stop.hide()
@@ -399,7 +401,8 @@ class ProgressDialog (ModalDialog):
             
     def setup_buttons (self, cancel, okay):                        
         # setup pause button 
-        self.pause = gtk.ToggleButton(_('_Pause'),True)        
+        self.pause = gtk.ToggleButton(_('_Pause'))
+        self.pause.set_use_underline(True)        
         self.action_area.pack_end(self.pause)
         # only show it/connect it if we want to...
         if self.custom_pausecb:
@@ -857,8 +860,15 @@ def saveas_file (title,
                            show_filetype=show_filetype,parent=parent)
     retval = sfd.run()
     if not retval:
-        return None,None    
-    exp_type = get_type_for_filters(retval, filters)
+        return None,None
+    exp_type = get_type_for_filters(retval, filters[:])
+    if not exp_type:
+        # If we don't have a type based on our regexps... then lets
+        # just see what the combobox was set to...
+        try:
+            exp_type = filters[sfd.saveas.get_active()][0]
+        except:
+            pass
     return retval, exp_type
 
 def get_type_for_filters (fname, filters):
