@@ -1,11 +1,12 @@
 #!/usr/bin/python
 import os,stat,re,time,StringIO
 from gourmet import keymanager, convert, ImageExtras
-from gourmet.gdebug import debug, TimeAction, print_timer_info
+from gourmet.gdebug import debug, TimeAction, print_timer_info, debug_decorator
 import gourmet.gglobals
 from gourmet.recipeManager import get_recipe_manager # for getting out database...
 import xml.sax.saxutils
 from gettext import gettext as _
+import gettext
 import gourmet.gtk_extras.dialog_extras as de
 import re
 from gourmet.threadManager import SuspendableThread, Terminated
@@ -87,13 +88,7 @@ class Importer (SuspendableThread):
         else:
             self.rating_converter = RatingConverter()
             self.do_conversion = True
-        
-        if hasattr(self.rd,'km'):
-            debug('Using existing keymanager',2)
-            self.km=self.rd.km
-        else:
-            debug('Making new keymanager',2)
-            self.km=keymanager.KeyManager(rm=self.rd)
+        self.km = keymanager.get_keymanager()
         timeaction.end()
         SuspendableThread.__init__(self,
                                    name=name)
@@ -169,13 +164,27 @@ class Importer (SuspendableThread):
         timeaction = TimeAction('importer.commit_rec',10)
         for key in ['cuisine','category','title']:
             if self.rec.has_key(key):
-                self.rec[key]=re.sub('\s+',' ',self.rec[key]).strip()
-        # if servings can't be recognized as a number, add them to the
-        # instructions.
+                self.rec[key]=unicode(re.sub('\s+',' ',self.rec[key]).strip())
+        # if yields/servings can't be recognized as a number, add them
+        # to the instructions.
+        if self.rec.has_key('yields'):
+            try:
+                self.rec['yields'] = float(self.rec['yields'])
+            except:
+                yields = self.convert_str_to_num(self.rec['yields'])
+                if not yields:
+                    print 'Moving yields to instructions!'
+                    self._move_to_instructions(self.rec,'yields')
+                else:
+                    self.rec['yields'] = yields
         if self.rec.has_key('servings'):
-            servs=self.convert_servings(self.rec['servings'])
+            servs=self.convert_str_to_num(self.rec['servings'])
             if servs != None:
-                self.rec['servings']=str(servs)
+                self.rec['servings'] = str(servs)
+                self.rec['yields'] = servs
+                self.rec['yield_unit'] = gettext.ngettext('serving',
+                                                          'servings',
+                                                          servs)
             else:
                 self._move_to_instructions(self.rec,'servings')
         # Check preptime and cooktime
@@ -257,9 +266,9 @@ class Importer (SuspendableThread):
                 _("Imported %s of %s recipes.")%(self.count,self.total)
                 )
                       
-    def convert_servings (self, str):
+    def convert_str_to_num (self, str):
         """Return a numerical servings value"""
-        timeaction = TimeAction('importer.convert_servings',10)
+        timeaction = TimeAction('importer.convert_str_to_num',10)
         debug('converting servings for %s'%str,5)
         try:
             return float(str)
@@ -287,7 +296,7 @@ class Importer (SuspendableThread):
         #    self.ing['recipe_id']=self.rec['id']
         #debug('ing ID %s, recipe ID %s'%(self.ing['recipe_id'],self.rec['id']),0)
         timeaction.end()
-                 
+
     def finish_ing (self):
         timeaction = TimeAction('importer.finish_ing 1',10)
         # Strip whitespace...
@@ -478,7 +487,7 @@ class RatingConverter:
         """
         self.got_conversions=True
         if not star_generator:
-            from gourmet.ratingWidget import star_generator
+            from gourmet.gtk_extras.ratingWidget import star_generator
         ratings = []
         need_conversions = False
         for v in self.to_convert.values():
