@@ -1,4 +1,5 @@
 import gtk, pango
+from xml.sax.saxutils import escape
 from gettext import gettext as _
 from generic_recipe_parser import RecipeParser
 import gourmet.gtk_extras.cb_extras as cb
@@ -19,15 +20,23 @@ import gourmet.ImageExtras as ImageExtras
 
 DEFAULT_TAGS = []
 DEFAULT_TAG_LABELS = gglobals.REC_ATTR_DIC.copy()
+DEFAULT_TAG_LABELS.update(gglobals.TEXT_ATTR_DIC)
 for attr in gglobals.DEFAULT_ATTR_ORDER:
     if attr != 'link':
         DEFAULT_TAGS.append(attr)
-DEFAULT_TAGS.extend(['instructions','ingredients','inggroup','ignore'])
-for tag,label in [('instructions',_('Instructions')),
-                  ('ingredients',_('Ingredients')),
-                  ('inggroup',_('Ingredient Subgroup')),
-                  ('ignore',_('Hide'))
-                  ]:
+        if attr == 'yields':
+            # a bit of a hack -- we want to make 'yield unit' a button
+            DEFAULT_TAGS.append('yield_unit')
+            DEFAULT_TAGS.append('servings')
+DEFAULT_TAGS.extend(gglobals.DEFAULT_TEXT_ATTR_ORDER)
+
+DEFAULT_TAGS.extend(['ingredients','inggroup','ignore'])
+for tag,label in [
+    ('servings',_('Servings')),
+    ('ingredients',_('Ingredients')),
+    ('inggroup',_('Ingredient Subgroup')),
+    ('ignore',_('Hide'))
+    ]:
     if not DEFAULT_TAG_LABELS.has_key(tag):
         DEFAULT_TAG_LABELS[tag] = label
 
@@ -69,16 +78,19 @@ class ConvenientImporter (importer.Importer):
             # abbreviations through)
             if unit and not self.conv.unit_dict.has_key(unit.strip()) and len(unit.strip())>2:
                 item = unit + ' ' + item
-                unit = ''                
+                unit = ''
         else:
             print 'Unable to parse ingredient from text "%s"'%txt
             print 'Setting amount and unit to None'
             amount = None; unit = None;
             item = txt
         self.start_ing()
-        if amount: self.add_amt(amount)
-        if unit: self.add_unit(unit)
-        if item: self.add_item(item)
+        if amount:
+            self.add_amt(amount)
+        if unit:
+            self.add_unit(unit)
+        if item:
+            self.add_item(item)
         self.commit_ing()
 
     def add_ings_from_text (self, txt, break_at='\n'):
@@ -86,7 +98,8 @@ class ConvenientImporter (importer.Importer):
 
         By default, there is one ingredient per line of text."""
         txt=txt.strip()
-        for i in txt.split(break_at): self.add_ing_from_text(i)
+        for i in txt.split(break_at):
+            self.add_ing_from_text(i)
 
 class InteractiveImporter (ConvenientImporter, NotThreadSafe):
 
@@ -96,10 +109,13 @@ class InteractiveImporter (ConvenientImporter, NotThreadSafe):
                   custom_parser = None,
                   tags=DEFAULT_TAGS,
                   tag_labels=DEFAULT_TAG_LABELS,
-                  modal=True):
+                  modal=True,
+                  title=_('Import recipe')):
+        self.title = title
         if custom_parser: self.parser = custom_parser
         else: self.parser = RecipeParser()
         self.labels_by_tag = tag_labels
+        print 'self.labels_by_tag=',self.labels_by_tag
         self.tags_by_label = {self.NEW_REC_TEXT:'newrec'}
         for k,v in self.labels_by_tag.items(): self.tags_by_label[v]=k
         self.tags = tags
@@ -114,7 +130,13 @@ class InteractiveImporter (ConvenientImporter, NotThreadSafe):
         ConvenientImporter.__init__(self)
         
     def setup_window (self):
-        self.w = gtk.Window()
+        # set our parent...
+        from gourmet.threadManager import get_thread_manager_gui
+        tmg = get_thread_manager_gui()
+        self.w = gtk.Window();
+        self.w.set_title(self.title)
+        self.w.set_transient_for(tmg.dialog)
+        self.w.set_destroy_with_parent(False)
         self.hb = gtk.HBox()
         self.w.add(self.hb)
         self.tv = gtk.TextView()
@@ -356,8 +378,7 @@ class InteractiveImporter (ConvenientImporter, NotThreadSafe):
             text = siter.get_text(eiter)
             name = smark.get_name()
             label = name.split('-')[0]
-            print 'Processing',label,'->',text
-            tag = self.tags_by_label[label]
+            tag = self.tags_by_label[label]            
             if tag in gglobals.TEXT_ATTR_DIC:
                 self.add_text(tag,text); started=True
             elif tag in gglobals.REC_ATTR_DIC:
@@ -377,10 +398,12 @@ class InteractiveImporter (ConvenientImporter, NotThreadSafe):
                 self.start_rec()
             elif tag=='ignore':
                 continue
+            elif tag == 'servings':
+                self.add_attribute('yields',text)
+                self.add_attribute('yield_unit','servings')
             else:
                 print 'UNKNOWN TAG',tag,text,label
         if started: self.commit_rec()
-        print "We've added the recipes...",self.added_recs
         if hasattr(self,'images') and self.images:
             # This is ugly -- we run the dialog once per recipe. This
             # should happen rarely in current use-case (I don't know
@@ -390,7 +413,7 @@ class InteractiveImporter (ConvenientImporter, NotThreadSafe):
             for rec in self.added_recs:
                 ibd = imageBrowser.ImageBrowserDialog(
                     title=_('Select recipe image'),
-                    label=_('Select image for recipe "%s"'%rec.title or _('Untitled')),
+                    label=_('Select image for recipe "%s"')%escape(rec.title or _('Untitled')),
                     sublabel=_("Below are all the images found for the page you are importing. Select any images that are of the recipe, or don't select anything if you don't want any of these images."),
                     )
                 for i in self.images: ibd.add_image_from_uri(i)
