@@ -1,3 +1,4 @@
+import gtk, gobject
 import reportlab
 from reportlab.pdfbase import pdfmetrics
 from reportlab.lib.units import inch,mm
@@ -13,7 +14,10 @@ from gettext import ngettext
 from gourmet import convert
 from gourmet import gglobals
 from gourmet.gtk_extras import dialog_extras as de
+from gourmet.gtk_extras import optionTable
+from gourmet.gtk_extras import cb_extras
 from gourmet import ImageExtras
+from gourmet.prefs import get_prefs
 import xml.sax.saxutils
 import gourmet.exporters.exporter as exporter
 import types, re
@@ -23,16 +27,7 @@ from page_drawer import PageDrawer
 
 PASS_REPORTLAB_UNICODE = (reportlab.Version.find('2')==0)
 
-DEFAULT_PDF_ARGS = {
-    #'pagesize':'letter',
-    #'pagemode':'landscape',
-    #'left_margin':0.3*inch,
-    #'right_margin':0.3*inch,
-    #'top_margin':0.3*inch,
-    #'bottom_margin':0.3*inch,
-    #'base_font_size':8,
-    #'mode':('index_cards',(5*inch,3.5*inch))
-    }
+DEFAULT_PDF_ARGS = {'bottom_margin': 72, 'pagesize': 'letter', 'right_margin': 72, 'top_margin': 72, 'left_margin': 72, 'pagemode': 'portrait', 'base_font_size': 10, 'mode': ('column', 1)}
 
 # Code for MCLine from:
 # http://two.pairlist.net/pipermail/reportlab-users/2005-February/003695.html
@@ -219,7 +214,7 @@ class Bookmark(platypus.Flowable):
 
 class PdfWriter:
 
-    def __init__ (self):
+    def __init__ (self, allrecs=[]):
         pass
 
     def setup_document (self, file, mode=('column',1), size='default', pagesize='letter',
@@ -406,10 +401,9 @@ class PdfExporter (exporter.exporter_mult, PdfWriter):
                   styleSheet=None,
                   txt=[],
                   pdf_args=DEFAULT_PDF_ARGS,
+                  all_recipes=[], # For learning about references...
                   **kwargs):
-        print 'Instantiate PdfExporter',doc,kwargs,txt
-        self.links = [] # Keep track of what recipes we link to to
-                        # make sure we use them...
+        self.all_recipes = all_recipes
         PdfWriter.__init__(self)
         if type(out) in types.StringTypes:
             self.out = file(out,'wb')
@@ -639,32 +633,27 @@ class PdfExporter (exporter.exporter_mult, PdfWriter):
             )
 
     def write_ingref (self, amount, unit, item, refid, optional):
-        reffed = self.rd.get_rec(refid)
-        if not reffed:
-            reffed = self.rd.fetch_one(self.rd.recipe_table,title=item,deleted=False)
-            if reffed:
-                refid = reffed.id
-        if not reffed:
+        if refid in [r.id for r in self.all_recipes]:
+            txt = ""
+            for blob in [amount,unit,item,(optional and _('optional') or '')]:
+                if blob == item:
+                    blob = '<link href="r%s">'%refid + blob + '</link>'  
+                elif not blob:
+                    continue
+                if txt: txt += " %s"%blob
+                else: txt = blob
+            hanging = inch*0.25
+            self.write_paragraph(
+                txt,
+                attributes=' firstLineIndent="-%(hanging)s" leftIndent="%(hanging)s"'%locals()
+                )
+        else:
             return self.write_ing(amount,unit,item,optional=optional)
-        txt = ""
-        for blob in [amount,unit,item,(optional and _('optional') or '')]:
-            if blob == item:
-              blob = '<link href="r%s">'%refid + blob + '</link>'  
-            if not blob: continue
-            if txt: txt += " %s"%blob
-            else: txt = blob
-        hanging = inch*0.25
-        self.links.append(refid)
-        self.write_paragraph(
-            txt,
-            attributes=' firstLineIndent="-%(hanging)s" leftIndent="%(hanging)s"'%locals()
-            )
 
 class PdfExporterMultiDoc (exporter.ExporterMultirec, PdfWriter):
     def __init__ (self, rd, recipes, out, progress_func=None, conv=None,
                   pdf_args=DEFAULT_PDF_ARGS,
                   **kwargs):
-        self.links = []
         PdfWriter.__init__(self)
         if type(out) in types.StringTypes:
             out = file(out,'wb')
@@ -674,6 +663,7 @@ class PdfExporterMultiDoc (exporter.ExporterMultirec, PdfWriter):
         kwargs['styleSheet'] = self.styleSheet
         kwargs['txt'] = self.txt
         kwargs['pdf_args'] = pdf_args
+        kwargs['all_recipes']=recipes
         exporter.ExporterMultirec.__init__(
             self,
             rd, recipes, out,
@@ -717,14 +707,110 @@ class PdfPageDrawer (PageDrawer):
 PDF_PREF_DEFAULT={
     'page_size':_('Letter'),
     'orientation':_('Portrait'),
-    'font_size':10.0,
+    'font_size':10,
     'page_layout':_('Plain'),
-    'left_margin':1.0,
-    'right_margin':1.0,
-    'top_margin':1.0,
-    'bottom_margin':1.0,    
+    'left_margin':1.0*inch,
+    'right_margin':1.0*inch,
+    'top_margin':1.0*inch,
+    'bottom_margin':1.0*inch,    
     }
 
+class CustomUnitOption (optionTable.CustomOption):
+
+    '''An option for optionTable with adjustable units -- used for margins.
+    '''
+
+    units = {
+        '"':inch,
+        _('cm'):10*mm,
+        _('points'):1,
+        }
+
+    min_val = 0.125*inch
+    max_val = 8*inch
+
+    adjustments = {
+        inch:(0.125,0.5),
+        10*mm:(0.5,1),
+        1:(5,25),
+        }
+
+    def __init__ (self, default_value = inch):
+        gobject.GObject.__init__(self)
+        gtk.HBox.__init__(self)
+        self.__quiet__ = False
+        self.unit_combo = gtk.combo_box_new_text()
+        for key in self.units:
+            self.unit_combo.append_text(key)
+        unit = get_prefs().get('default_margin_unit',_('cm'))
+        if unit not in self.units: unit = _('cm')
+        self.last_unit = self.units[unit]
+        cb_extras.setup_typeahead(self.unit_combo)
+        cb_extras.cb_set_active_text(self.unit_combo,unit)
+        self.unit_combo.connect('changed',self.unit_changed_cb)
+        self.value_adjustment = gtk.Adjustment(
+            value=self.adjust_to_unit(default_value),
+            lower= self.min_val / self.last_unit,
+            upper = self.max_val / self.last_unit,
+            step_incr = self.adjustments[self.last_unit][0],
+            page_incr = self.adjustments[self.last_unit][1],
+            )
+        def emit_changed (*args):
+            self.emit('changed')
+        self.value_adjustment.connect('changed',emit_changed)
+        self.value_widget = gtk.SpinButton(self.value_adjustment,digits=2)
+        self.value_widget.connect('changed',emit_changed)
+        self.value_widget.show(); self.unit_combo.show()
+        self.pack_start(self.value_widget)
+        self.pack_start(self.unit_combo)
+
+    def set_unit (self, unit):
+        cb_extras.cb_set_active_text(self.unit_combo,unit)
+        
+    def unit_changed_cb (self, widget):
+        new_unit = self.units[self.unit_combo.get_active_text()]
+        get_prefs()['default_margin_unit'] = self.unit_combo.get_active_text()
+        old_val = self.value_adjustment.get_value() * self.last_unit
+        self.last_unit = self.units[self.unit_combo.get_active_text()]
+        new_val = self.adjust_to_unit(old_val)
+        self.value_adjustment.set_upper(self.max_val / self.last_unit)
+        self.value_adjustment.set_lower(self.min_val / self.last_unit)
+        self.value_adjustment.set_step_increment(self.adjustments[self.last_unit][0])
+        self.value_adjustment.set_page_increment(self.adjustments[self.last_unit][1])
+        self.value_adjustment.set_value(new_val)
+        if not self.__quiet__:
+            self.emit('changed')
+
+    def adjust_to_unit (self, raw_val):
+        '''Round the value to an appropriate number for our current
+        unit
+        '''
+        val = raw_val / self.last_unit
+        adj = self.adjustments[self.last_unit][0]
+        # "Round" to the increment adjustment specified for our unit
+        floor =  int(val/adj) * adj
+        ceiling = (int(val/adj)+1) * adj
+        # Pick whatever is closest...
+        if abs(floor - val) > abs(ceiling - val):
+            return ceiling
+        else:
+            return floor
+        
+    def get_value (self):
+        return self.last_unit * self.value_adjustment.get_value()
+        
+    def set_value (self, value):
+        self.value_adjustment.set_value(value/self.last_unit)
+        if not self.__quiet__:
+            self.emit('changed')
+
+    def sync_to_other_cuo (self, cuo):
+        def change_cb (other_cuo):
+            self.__quiet__ = True
+            self.set_unit(other_cuo.unit_combo.get_active_text())
+            self.__quiet__ = False
+        cuo.connect('changed',change_cb)
+
 class PdfPrefGetter:
     page_sizes = {
         _('11x17"'):'elevenSeventeen',
@@ -756,44 +842,65 @@ class PdfPrefGetter:
         }
 
     OPT_PS,OPT_PO,OPT_FS,OPT_PL,OPT_LM,OPT_RM,OPT_TM,OPT_BM = range(8)
-    
-    def __init__ (self, defaults=PDF_PREF_DEFAULT):
+
+    def __init__ (self,):
+        self.prefs = get_prefs()
+        defaults = self.prefs.get('PDF_EXP',PDF_PREF_DEFAULT)
         self.size_strings = self.page_sizes.keys()
         self.size_strings.sort()
         for n in range(2,5):
             self.layouts[ngettext('%s Column','%s Columns',n)%n]=('column',n)
+        self.make_reverse_dicts()        
         self.layout_strings = self.layouts.keys()
         self.layout_strings.sort()
+        margin_widgets = [
+            CustomUnitOption(defaults.get(pref,PDF_PREF_DEFAULT[pref]))
+            for pref in ['left_margin','right_margin','top_margin','bottom_margin']
+            ]
+        # Make unit changes to one widget affect all the others!
+        for m in margin_widgets:
+            for mm in margin_widgets:
+                if mm is not m:
+                    m.sync_to_other_cuo(mm)
+            
         self.opts = [
             [_('Paper _Size')+':',(defaults.get('page_size',PDF_PREF_DEFAULT['page_size']),
                                   self.size_strings)],
             [_('_Orientation')+':',(defaults.get('orientation',PDF_PREF_DEFAULT['orientation']),
                                     self.page_modes.keys())],
-            [_('_Font Size')+':',defaults.get('font_size',PDF_PREF_DEFAULT['font_size'])]
+            [_('_Font Size')+':',int(defaults.get('font_size',PDF_PREF_DEFAULT['font_size']))]
             ,
             [_('Page _Layout'),(defaults.get('page_layout',PDF_PREF_DEFAULT['page_layout']),
 
                                 self.layout_strings)],
-            [_('Left Margin')+':',defaults.get('left_margin',PDF_PREF_DEFAULT['left_margin'])]
-            ,
-            [_('Right Margin')+':',defaults.get('right_margin',PDF_PREF_DEFAULT['right_margin'])]
-            ,
-            [_('Top Margin')+':',defaults.get('top_margin',PDF_PREF_DEFAULT['top_margin'])]
-            ,
-            [_('Bottom Margin')+':',defaults.get('bottom_margin',PDF_PREF_DEFAULT['bottom_margin'])]
-            ,
+            [_('Left Margin')+':',margin_widgets[0]],
+            [_('Right Margin')+':',margin_widgets[1]],
+            [_('Top Margin')+':',margin_widgets[2]],
+            [_('Bottom Margin')+':',margin_widgets[3]],
             ]
         
         self.page_drawer = PdfPageDrawer(yalign=0.0)    
         self.in_ccb = False
+        self.setup_widgets()
+        self.table.connect('changed',self.change_cb)
+        self.table.emit('changed')
+        self.page_drawer.set_size_request(200,100)
+        self.page_drawer.show()
+
+    def make_reverse_dicts (self):
+        self.page_sizes_r = {}; self.layouts_r = {}; self.page_modes_r = {}
+        for dict,dict_r in [
+            (self.page_sizes,self.page_sizes_r),
+            (self.layouts,self.layouts_r),
+            (self.page_modes,self.page_modes_r)]:
+            for k,v in dict.items(): dict_r[v]=k
+
+    def setup_widgets (self):
         self.pd = de.PreferencesDialog(self.opts,option_label=None,value_label=None,
                                   label='PDF Options',
                                   )
-        self.pd.table.connect('changed',self.change_cb)
-        self.pd.table.emit('changed')
-        self.pd.hbox.pack_start(self.page_drawer,fill=True,expand=True)        
-        self.page_drawer.set_size_request(200,100)
-        self.page_drawer.show()
+        self.pd.hbox.pack_start(self.page_drawer,fill=True,expand=True)
+        self.table = self.pd.table
 
     def run (self):
         self.pd.run()
@@ -801,14 +908,20 @@ class PdfPrefGetter:
 
     def get_args_from_opts (self, opts):
         args = {}
-        args['pagesize']=self.page_sizes[opts[self.OPT_PS][1]] # PAGE SIZE
-        args['pagemode']=self.page_modes[opts[self.OPT_PO][1]] # PAGE MODE
-        args['base_font_size']=opts[self.OPT_FS][1] # FONT SIZE
-        args['mode']=self.layouts[opts[self.OPT_PL][1]] # LAYOUT/MODE
-        args['left_margin']=opts[self.OPT_LM][1]*inch
-        args['right_margin']=opts[self.OPT_RM][1]*inch
-        args['top_margin']=opts[self.OPT_TM][1]*inch
-        args['bottom_margin']=opts[self.OPT_BM][1]*inch
+        if not get_prefs().has_key('PDF_EXP'):
+            get_prefs()['PDF_EXP'] = {}
+        prefs = get_prefs()['PDF_EXP']
+        args['pagesize'] = self.page_sizes[opts[self.OPT_PS][1]] # PAGE SIZE
+        prefs['page_size'] = self.page_sizes_r[args['pagesize']]
+        args['pagemode'] = self.page_modes[opts[self.OPT_PO][1]] # PAGE MODE
+        prefs['orientation'] = self.page_modes_r[args['pagemode']]
+        prefs['font_size'] = args['base_font_size'] = opts[self.OPT_FS][1] # FONT SIZE
+        args['mode'] = self.layouts[opts[self.OPT_PL][1]] # LAYOUT/MODE
+        prefs['page_layout'] = self.layouts_r[args['mode']]
+        prefs['left_margin'] = args['left_margin'] = opts[self.OPT_LM][1]
+        prefs['right_margin'] = args['right_margin'] = opts[self.OPT_RM][1]
+        prefs['top_margin'] = args['top_margin'] = opts[self.OPT_TM][1]
+        prefs['bottom_margin'] = args['bottom_margin'] = opts[self.OPT_BM][1]
         return args
 
     def change_cb (self, option_table, *args,**kwargs):
@@ -892,11 +1005,44 @@ class PdfPrefGetter:
         self.page_drawer.queue_draw()
         self.in_ccb = False
 
-def get_pdf_prefs (defaults=PDF_PREF_DEFAULT):
-    pdf_pref_getter = PdfPrefGetter(defaults=defaults)
+class PdfPrefTable (PdfPrefGetter):
+
+    # Like the dialog, but without the window -- lets it be embedded
+    # in a print preferences widget.
+
+    def setup_widgets (self):
+        self.widg = gtk.HBox()
+        self.table = optionTable.OptionTable(options=self.opts,
+                                             option_label=None,
+                                             value_label=None,
+                                             changedcb=None)
+        self.widg.pack_start(self.table)
+        self.widg.pack_start(self.page_drawer,fill=True,expand=True)
+        self.widg.show_all()
+
+def get_pdf_prefs (defaults=None):
+    if defaults: print 'WARNING: ignoring provided defaults and using prefs system instead'
+    pdf_pref_getter = PdfPrefGetter()
     return pdf_pref_getter.run()
             
 if __name__ == '__main__':
+    w = gtk.Window()
+    cuo = CustomUnitOption(44)
+    cuo2 = CustomUnitOption(98)
+    cuo.sync_to_other_cuo(cuo2)
+    cuo2.sync_to_other_cuo(cuo)
+    vb = gtk.VBox()
+    l = gtk.Label('Hello World')
+    vb.add(l)
+    vb.pack_start(cuo)
+    vb.pack_start(cuo2)
+    w.add(vb)
+    vb.show(); cuo.show(); cuo2.show()
+    w.show()
+    w.connect('delete_event',gtk.main_quit)
+    gtk.main()
+    raise 'Hell'
+    
     from tempfile import tempdir
     import os.path
     #opts = get_pdf_prefs(); print opts
@@ -961,7 +1107,6 @@ if __name__ == '__main__':
     #pe.run()
 
     def test_formatting ():
-        print 'Test formatting'
         sw = PdfWriter()
         f = file(os.path.join(tempdir,'format.pdf'),'wb')
         sw.setup_document(f)
