@@ -5,6 +5,7 @@ from gourmet.gtk_extras import cb_extras as cb
 from gourmet.gtk_extras import dialog_extras as de
 from gettext import gettext as _
 from gettext import ngettext
+import keyEditorPluggable
 #import nutrition.nutritionDruid as nutritionDruid
 
 try:
@@ -45,7 +46,9 @@ class KeyEditor:
         # Make our lovely model
         self.makeTreeModel()
         # setup completion in entry
-        cb.make_completion(self.changeKeyEntry,self.rg.inginfo.key_model)
+        model = gtk.ListStore(str)
+        for k in self.rd.get_unique_values('ingkey',table=self.rd.ingredients_table): model.append([k])
+        cb.make_completion(self.changeKeyEntry,model)
         # Setup next/prev/first/last buttons for view
         self.prev_button = self.glade.get_widget('prevButton')
         self.next_button = self.glade.get_widget('nextButton')
@@ -58,7 +61,7 @@ class KeyEditor:
         self.last_button.connect('clicked',lambda *args: self.treeModel.goto_last_page())
         # Setup search stuff
         self.search_string=""
-        self.search_by = _('Key')
+        self.search_by = _('key')
         self.use_regexp=True
         self.setupTreeView()
         self.treeview.set_model(self.treeModel)
@@ -137,7 +140,11 @@ class KeyEditor:
                 col.set_property('expand',True)
             col.set_resizable(True)
             self.treeview.append_column(col)
-
+        plugin_manager = keyEditorPluggable.get_key_editor_plugin_manager()
+        for tvc in plugin_manager.get_treeview_columns(None,
+                                                       key_col=2,
+                                                       instant_apply=True):
+            self.treeview.append_column(tvc)
 
     def tree_edited (self, renderer, path_string, text, n, head):
         indices = path_string.split(':')
@@ -151,7 +158,7 @@ class KeyEditor:
             if de.getBoolean(label=_('Change all keys "%s" to "%s"?')%(key,text),
                              sublabel=_("You won't be able to undo this action. If there are already ingredients with the key \"%s\", you won't be able to distinguish between those items and the items you are changing now."%text)
                              ):
-                self.rd.update(
+                self.rd.update_by_criteria(
                     self.rd.ingredients_table,
                     curdic,
                     {'ingkey':text}
@@ -164,7 +171,7 @@ class KeyEditor:
             if de.getBoolean(label=_('Change all items "%s" to "%s"?')%(curdic['item'],text),
                              sublabel=_("You won't be able to undo this action. If there are already ingredients with the item \"%s\", you won't be able to distinguish between those items and the items you are changing now.")%text
                              ):
-                self.rd.update(
+                self.rd.update_by_criteria(
                     self.rd.ingredients_table,
                     curdic,
                     {'item':text}
@@ -179,13 +186,13 @@ class KeyEditor:
                               default = 2,
                               )
             if val==1:
-                self.rd.update(
+                self.rd.update_by_criteria(
                     self.rd.ingredients_table,
                     {'unit':unit},
                     {'unit':text},
                     )
             elif val==2:
-                self.rd.update(
+                self.rd.update_by_criteria(
                     self.rd.ingredients_table,
                     curdic,
                     {'unit':text}
@@ -211,7 +218,7 @@ class KeyEditor:
                 cond = {'unit':unit,'amount':amount,'ingkey':key}
             elif val == 3:
                 cond = curdic
-            self.rd.update(
+            self.rd.update_by_criteria(
                 self.rd.ingredients_table,
                 {'unit':unit,'amount':convert.frac_to_float(amount)},
                 {'unit':unit,'amount':new_amount}
@@ -428,7 +435,6 @@ class KeyEditor:
                 nid.show()
             
     def update_nutinfo (self, *args):
-        print 'KeyEditor.update_nutinfo'
         self.treeModel.reset_views()
 
     def update_iter (self, itr, newdic):
@@ -508,6 +514,7 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
     
     columns = ['obj','ingkey','item','count','recipe']#,'ndbno']
     def __init__ (self, rd, per_page=15):
+        self.__last_limit_text = ''
         self.rd = rd
         pageable_store.PageableTreeStore.__init__(self,
                                                   [gobject.TYPE_PYOBJECT, # row ref
@@ -527,7 +534,6 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
             try:
                 itr = self.get_iter(path)
             except ValueError:
-                print 'Trouble with path',path
                 return
             self.emit('row-changed',path,itr)
             child = self.iter_children(itr)
@@ -551,6 +557,10 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
         self.limit(txt,'item',search_options)
 
     def limit (self, txt, column='ingkey', search_options={}):
+        if txt == self.__last_limit_text:
+            return
+        else:
+            self.__last_limit_text = txt
         if not txt: self.reset_views()
         if search_options['use_regexp']:
             s = {'search':txt,'operator':'REGEXP'}
@@ -654,7 +664,7 @@ class KeyStore (pageable_store.PageableTreeStore,pageable_store.PageableViewStor
 
     def get_recs (self, key, item):
         """Return a string with a list of recipes containing an ingredient with key and item"""
-        recs = [i.id for i in self.rd.fetch_all(self.rd.ingredients_table,ingkey=key,item=item)]
+        recs = [i.recipe_id for i in self.rd.fetch_all(self.rd.ingredients_table,ingkey=key,item=item)]
         titles = []
         looked_at = []
         for r_id in recs:
