@@ -46,7 +46,8 @@ class NutritionData:
         if prev_association:
             self.db.do_modify(self.db.nutritionaliases_table,
                               prev_association,
-                              {'ndbno':ndbno})
+                              {'ndbno':ndbno},
+                              "ingkey")
         else:
             self.db.do_add(self.db.nutritionaliases_table,{'ndbno':ndbno,
                                                  'ingkey':key}
@@ -103,7 +104,11 @@ class NutritionData:
             return self.get_nutinfo_for_inglist(rd.get_ings(subrec),rd,ingObject=ing)
         if hasattr(ing,'rangeamount') and ing.rangeamount:
             # just average our amounts
-            amount = (ing.rangeamount + ing.amount)/2
+            try:
+                amount = (ing.rangeamount + ing.amount)/2
+            except TypeError:
+                print 'Failed trying to add',ing.rangeamount,ing.amount
+                raise
         else:
             amount = ing.amount
         if not amount: amount=1
@@ -608,10 +613,8 @@ def foo ():
             print att,":",getattr(self.ings,att)
             vv = self.ings._get_vapor()
             if vv:
-                print '(but we have some vapor)'
                 for v in vv:
                     explanation = v._wheres_the_vapor()
-                    print 'Vapor for ',v.__key__
                     if explanation==KEY_VAPOR: print 'No key'
                     if explanation==UNIT_VAPOR: print "Can't handle unit ",v.__unit__
                     if explanation==AMOUNT_VAPOR: print "What am I to do with the amount ",v.__amt__
