@@ -22,7 +22,6 @@ time_matcher = re.compile(
 
 
 def make_time_links (s):
-    s = xml.sax.saxutils.escape(s)
     return time_matcher.sub('<a href="\g<firstnum> \g<unit>">\g<0></a>',s)
 
 
@@ -70,7 +69,7 @@ if gtk.pygtk_version[1] < 8:
 if __name__ == '__main__':
     
     import gtk
-    c = convert.Converter()
+    c = convert.get_converter()
     tv = LinkedTimeView()
     tv.connect('time-link-activated',show_timer_cb,c)
     tv.get_buffer().set_text(
